const svcNS      = require('../../definition.js').svcNS;
const EppError   = require('../../common/EppError.js');
const element    = require('../../common/element.js');
const token      = require('../../common/token.js');
const postalInfo_= require('../../common/contact/postalInfo.js');
const e164_      = require('../../common/contact/e164.js');
const authInfo_  = require('../../common/contact/authInfo.js');
const disclose_  = require('../../common/contact/disclose.js');

function epp2json(node) {
    let _id = undefined, _postalInfo = [], _voice = undefined, _fax = undefined,
        _email = undefined, _authInfo = undefined, _disclose = [],
        create = element.array(node);
    for(let i=0; i<create.length; i++)
        switch(create[i][0]) {
        case 'contact:id':
            element.limit(create[i], _id!==undefined);
            _id = create[i];
            break;
        case 'contact:postalInfo':
            element.limit(create[i], _postalInfo.length, 2);
            _postalInfo.push(create[i]);
            break;
        case 'contact:voice':
            element.limit(create[i], _voice!==undefined);
            _voice = create[i];
            break;
        case 'contact:fax':
            element.limit(create[i], _fax!==undefined);
            _fax = create[i];
            break;
        case 'contact:email':
            element.limit(create[i], _email!==undefined);
            _email = create[i];
            break;
        case 'contact:authInfo':
            element.limit(create[i], _authInfo!==undefined);
            _authInfo = create[i];
            break;
        case 'contact:disclose':
            element.limit(create[i], _disclose.length, 2);
            _disclose.push(create[i]);
            break;
        default:
            throw new EppError(create[i], create[i][0]+': unknown');
        }
    element.exist(node, 'contact:id', _id!==undefined);
    element.exist(node, 'contact:email', _email!==undefined);
    element.exist(node, 'contact:authInfo', _authInfo!==undefined);

    let json = {
        command: 'create',
        obj: 'contact',
        id: token("{3,16}").epp2json(_id)
    }
    if(_postalInfo.length)
        json.postalInfo = postalInfo_.epp2json(_postalInfo);
    if(_voice!==undefined)
        json.voice = e164_.epp2json(_voice);
    if(_fax!==undefined)
        json.fax = e164_.epp2json(_fax);
    json.email = token("+").epp2json(_email);
    json.authInfo = authInfo_.epp2json(_authInfo);
    if(_disclose.length)
        json.disclose = disclose_.epp2json(_disclose);
    return json;
}

function json2epp(json) {
    let create = [
        ['contact:id', {}, token("{3,16}").json2epp(json, 'id')]
    ]
    if(json.postalInfo!==undefined)
        create.push(...postalInfo_.json2epp(json.postalInfo));
    if(json.voice!==undefined)
        create.push(e164_.json2epp(json, 'voice'));
    if(json.fax!==undefined)
        create.push(e164_.json2epp(json, 'fax'));
    create.push(['contact:email', {}, token("+").json2epp(json, 'email')]);
    create.push(authInfo_.json2epp(element.value(json, 'authInfo')));
    if(json.disclose!==undefined)
        create.push(...disclose_.json2epp(json.disclose));
    return create;
}

module.exports = {
    epp2json,
    json2epp: json => ["contact:create", {"xmlns:contact": svcNS.contact}, json2epp(json)]
}
