const svcNS      = require('../../definition.js').svcNS;
const EppError   = require('../../common/EppError.js');
const element    = require('../../common/element.js');
const token      = require('../../common/token.js');
const postalInfo_= require('../../common/contact/postalInfo.js');
const e164_      = require('../../common/contact/e164.js');
const authInfo_  = require('../../common/contact/authInfo.js');
const disclose_  = require('../../common/contact/disclose.js');
const status_    = require('./status.js');

function addRem2json(node) {
    let _status = [],
        addRem = element.array(node);
    for(let i=0; i<addRem.length; i++)
        _status.push(element.tag(addRem[i], ['contact:status']));
    return {status: status_.epp2json(_status)}
}

function json2addRem(json) {
    let addRem = [];
    if(json.status instanceof Array)
        addRem.push(...status_.json2epp(json));
    return addRem.length ? addRem : undefined;
}

function chg2json(node) {
    let _postalInfo = [], _voice = undefined, _fax = undefined,
        _email = undefined, _authInfo = undefined, _disclose = [],
        chg = element.array(node);
    for(let i=0; i<chg.length; i++)
        switch(chg[i][0]) {
        case 'contact:postalInfo':
            element.limit(chg[i], _postalInfo.length, 2);
            _postalInfo.push(chg[i]);
            break;
        case 'contact:voice':
            element.limit(chg[i], _voice!==undefined);
            _voice = chg[i];
            break;
        case 'contact:fax':
            element.limit(chg[i], _fax!==undefined);
            _fax = chg[i];
            break;
        case 'contact:email':
            element.limit(chg[i], _email!==undefined);
            _email = chg[i];
            break;
        case 'contact:authInfo':
            element.limit(chg[i], _authInfo!==undefined);
            _authInfo = chg[i];
            break;
        case 'contact:disclose':
            element.limit(chg[i], _disclose.length, 2);
            _disclose.push(chg[i]);
            break;
        default:
            throw new EppError(chg[i], chg[i][0]+': unknown');
        }

    let json = {};
    if(_postalInfo.length)
        json.postalInfo = postalInfo_.epp2json(_postalInfo);
    if(_voice!==undefined)
        json.voice = e164_.epp2json(_voice);
    if(_fax!==undefined)
        json.fax = e164_.epp2json(_fax);
    if(_email!==undefined)
        json.email = token("+").epp2json(_email);
    if(_authInfo!==undefined)
        json.authInfo = authInfo_.epp2json(_authInfo);
    if(_disclose.length)
        json.disclose = disclose_.epp2json(_disclose);
    return json;
}

function json2chg(json) {
    let chg = [];
    if(json.postalInfo!==undefined)
        chg.push(...postalInfo_.json2epp(json.postalInfo));
    if(json.voice!==undefined)
        chg.push(e164_.json2epp(json, 'voice'));
    if(json.fax!==undefined)
        chg.push(e164_.json2epp(json, 'fax'));
    if(json.email!==undefined)
        chg.push(['contact:email', {}, token("+").json2epp(json, 'email')]);
    if(json.authInfo!==undefined)
        chg.push(authInfo_.json2epp(json.authInfo));
    if(json.disclose!==undefined)
        chg.push(...disclose_.json2epp(json.disclose));
    return chg;
}

function epp2json(node) {
    let _id = undefined, _add = undefined, _rem = undefined, _chg = undefined,
        update = element.array(node);
    for(let i=0; i<update.length; i++)
        switch(update[i][0]) {
        case 'contact:id':
            element.limit(update[i], _id!==undefined);
            _id = update[i];
            break;
        case 'contact:add':
            element.limit(update[i], _add!==undefined);
            _add = update[i];
            break;
        case 'contact:rem':
            element.limit(update[i], _rem!==undefined);
            _rem = update[i];
            break;
        case 'contact:chg':
            element.limit(update[i], _chg!==undefined);
            _chg = update[i];
            break;
        default:
            throw new EppError(update[i], update[i][0]+': unknown');
        }
    element.exist(node, 'contact:id', _id!==undefined);

    let json = {
        command: 'update',
        obj: 'contact',
        id: token("{3,16}").epp2json(_id)
    }
    if(_add!==undefined)
        json.add = addRem2json(_add);
    if(_rem!==undefined)
        json.rem = addRem2json(_rem);
    if(_chg!==undefined)
        json.chg = chg2json(_chg);
    return json;
}

function json2epp(json) {
    let update = [
        ['contact:id', {}, token("{3,16}").json2epp(json, 'id')]
    ]
    if(json.add!==undefined)
        update.push(['contact:add', {}, json2addRem(json.add)]);
    if(json.rem!==undefined)
        update.push(['contact:rem', {}, json2addRem(json.rem)]);
    if(json.chg!==undefined)
        update.push(['contact:chg', {}, json2chg(json.chg)]);
    return update;
}

module.exports = {
    epp2json,
    json2epp: json => ["contact:update", {"xmlns:contact": svcNS.contact}, json2epp(json)]
}
