const svcNS    = require('../../definition.js').svcNS;
const EppError = require('../../common/EppError.js');
const element  = require('../../common/element.js');
const label_   = require('../../common/label.js');
const token    = require('../../common/token.js');
const period_  = require('../../common/domain/period.js');
const ns_      = require('../../common/domain/ns.js');
const contact_ = require('../../common/domain/contact.js');
const authInfo_= require('../../common/domain/authInfo.js');

function epp2json(node) {
    let _name = undefined, _period = undefined, _ns = undefined,
        _registrant = undefined, _contact = [], _authInfo = undefined,
        create = element.array(node);
    for(let i=0; i<create.length; i++)
        switch(create[i][0]) {
        case 'domain:name':
            element.limit(create[i], _name!==undefined);
            _name = create[i];
            break;
        case 'domain:period':
            element.limit(create[i], _period!==undefined);
            _period = create[i];
            break;
        case 'domain:ns':
            element.limit(create[i], _ns!==undefined);
            _ns = create[i];
            break;
        case 'domain:registrant':
            element.limit(create[i], _registrant!==undefined);
            _registrant = create[i];
            break;
        case 'domain:contact':
            _contact.push(create[i]);
            break;
        case 'domain:authInfo':
            element.limit(create[i], _authInfo!==undefined);
            _authInfo = create[i];
            break;
        default:
            throw new EppError(create[i], create[i][0]+': unknown');
        }
    element.exist(node, 'domain:name', _name!==undefined);
    element.exist(node, 'domain:authInfo', _authInfo!==undefined);

    let json = {
        command: 'create',
        obj: 'domain',
        name: label_.epp2json(_name)
    }
    if(_period!==undefined)
        json.period = period_.epp2json(_period);
    if(_ns!==undefined)
        json.ns = ns_.epp2json(_ns);
    if(_registrant!==undefined)
        json.registrant = token("{3,16}").epp2json(_registrant);
    if(_contact.length)
        json.contact = contact_.epp2json(_contact);
    json.authInfo = authInfo_.epp2json(_authInfo);
    return json;
}

function json2epp(json) {
    let create = [
        ['domain:name', {}, label_.json2epp(json, 'name')]
    ]
    if(json.period!==undefined)
        create.push(period_.json2epp('domain', json.period));
    if(json.ns!==undefined)
        create.push(ns_.json2epp(json));
    if(json.registrant!==undefined)
        create.push(['domain:registrant', {}, token("{3,16}").json2epp(json, 'registrant')]);
    if(json.contact!==undefined)
        create.push(...contact_.json2epp(json.contact));
    create.push(authInfo_.json2epp(element.value(json, 'authInfo')));
    return create;
}

module.exports = {
    epp2json,
    json2epp: json => ["domain:create", {"xmlns:domain": svcNS.domain}, json2epp(json)]
}
