const svcNS    = require('../../definition.js').svcNS;
const EppError = require('../../common/EppError.js');
const element  = require('../../common/element.js');
const label_   = require('../../common/label.js');
const authInfo_= require('../../common/domain/authInfo.js');
const HOSTS    = ['all', 'del', 'none', 'sub']

function epp2json(node) {
    let _name = undefined, _authInfo = undefined,
        info = element.array(node);
    for(let i=0; i<info.length; i++)
        switch(info[i][0]) {
        case 'domain:name':
            element.limit(info[i], _name!==undefined);
            _name = info[i];
            break;
        case 'domain:authInfo':
            element.limit(info[i], _authInfo!==undefined);
            _authInfo = info[i];
            break;
        default:
            throw new EppError(info[i], info[i][0]+': unknown');
        }
    element.exist(node, 'domain:name', _name!==undefined);

    let json = {
        command: 'info',
        obj: 'domain',
        name: label_.epp2json(_name)
    }
    if(_name[1].hosts!==undefined)
        json.hosts = element.attribute(_name, 'hosts', HOSTS);
    if(_authInfo!==undefined)
        json.authInfo = authInfo_.epp2json(_authInfo);
    return json;
}

function json2epp(json) {
    let attrs = {};
    if(json.hosts!==undefined)
        attrs.hosts = element.value(json, 'hosts', HOSTS);

    let info = [
        ['domain:name', attrs, label_.json2epp(json, 'name')]
    ]
    if(json.authInfo!==undefined)
        info.push(authInfo_.json2epp(json.authInfo));
    return info;
}

module.exports = {
    epp2json,
    json2epp: json => ["domain:info", {"xmlns:domain": svcNS.domain}, json2epp(json)]
}
