const svcNS    = require('../../definition.js').svcNS;
const EppError = require('../../common/EppError.js');
const element  = require('../../common/element.js');
const label_   = require('../../common/label.js');
const period_  = require('../../common/domain/period.js');
const patt     = /^\d{4}-\d{2}-\d{2}$/;

function date2json(curExpDate) {
    if(!patt.test(element.string(curExpDate)))
        throw new EppError(curExpDate, curExpDate[0]+': '+patt);
    return curExpDate[2];
}

function json2date(json, key) {
    let s = element.value(json, key);
    if(!patt.test(s))
        throw new EppError(json, key+': '+patt);
    return s;
}

function epp2json(node) {
    let _name = undefined, _curExpDate = undefined, _period = undefined,
        renew = element.array(node);
    for(let i=0; i<renew.length; i++)
        switch(renew[i][0]) {
        case 'domain:name':
            element.limit(renew[i], _name!==undefined);
            _name = renew[i];
            break;
        case 'domain:curExpDate':
            element.limit(renew[i], _curExpDate!==undefined);
            _curExpDate = renew[i];
            break;
        case 'domain:period':
            element.limit(renew[i], _period!==undefined);
            _period = renew[i];
            break;
        default:
            throw new EppError(renew[i], renew[i][0]+': unknown');
        }
    element.exist(node, 'domain:name', _name!==undefined);
    element.exist(node, 'domain:curExpDate', _curExpDate!==undefined);

    let json = {
        command: 'renew',
        obj: 'domain',
        name: label_.epp2json(_name),
        curExpDate: date2json(_curExpDate)
    }
    if(_period!==undefined)
        json.period = period_.epp2json(_period);
    return json;
}

function json2epp(json) {
    let renew = [
        ['domain:name', {}, label_.json2epp(json, 'name')],
        ['domain:curExpDate', {}, json2date(json, 'curExpDate')]
    ]
    if(json.period!==undefined)
        renew.push(period_.json2epp('domain', json.period));
    return renew;
}

module.exports = {
    epp2json,
    json2epp: json => ["domain:renew", {"xmlns:domain": svcNS.domain}, json2epp(json)]
}
