const svcNS    = require('../../definition.js').svcNS;
const EppError = require('../../common/EppError.js');
const element  = require('../../common/element.js');
const label_   = require('../../common/label.js');
const period_  = require('../../common/domain/period.js');
const authInfo_= require('../../common/domain/authInfo.js');

function epp2json(node) {
    let _name = undefined, _period = undefined, _authInfo = undefined,
        transfer = element.array(node);
    for(let i=0; i<transfer.length; i++)
        switch(transfer[i][0]) {
        case 'domain:name':
            element.limit(transfer[i], _name!==undefined);
            _name = transfer[i];
            break;
        case 'domain:period':
            element.limit(transfer[i], _period!==undefined);
            _period = transfer[i];
            break;
        case 'domain:authInfo':
            element.limit(transfer[i], _authInfo!==undefined);
            _authInfo = transfer[i];
            break;
        default:
            throw new EppError(transfer[i], transfer[i][0]+': unknown');
        }
    element.exist(node, 'domain:name', _name!==undefined);

    let json = {
        command: 'transfer',
        obj: 'domain',
        name: label_.epp2json(_name)
    }
    if(_period!==undefined)
        json.period = period_.epp2json(_period);
    if(_authInfo!==undefined)
        json.authInfo = authInfo_.epp2json(_authInfo);
    return json;
}

function json2epp(json) {
    let transfer = [
        ['domain:name', {}, label_.json2epp(json, 'name')]
    ]
    if(json.period!==undefined)
        transfer.push(period_.json2epp('domain', json.period));
    if(json.authInfo!==undefined)
        transfer.push(authInfo_.json2epp(json.authInfo));
    return transfer;
}

module.exports = {
    epp2json,
    json2epp: json => ["domain:transfer", {"xmlns:domain": svcNS.domain}, json2epp(json)]
}
