const svcNS    = require('../../definition.js').svcNS;
const EppError = require('../../common/EppError.js');
const element  = require('../../common/element.js');
const label_   = require('../../common/label.js');
const token    = require('../../common/token.js');
const ns_      = require('../../common/domain/ns.js');
const contact_ = require('../../common/domain/contact.js');
const authInfo_= require('../../common/domain/authInfo.js');
const status_  = require('./status.js');

function addRem2json(node) {
    let _ns = undefined, _contact = [], _status = [],
        addRem = element.array(node);
    for(let i=0; i<addRem.length; i++)
        switch(addRem[i][0]) {
        case 'domain:ns':
            element.limit(addRem[i], _ns!==undefined);
            _ns = addRem[i];
            break;
        case 'domain:contact':
            _contact.push(addRem[i]);
            break;
        case 'domain:status':
            _status.push(addRem[i]);
            break;
        default:
            throw new EppError(addRem[i], addRem[i][0]+': unknown');
        }

    let json = {};
    if(_ns!==undefined)
        json.ns = ns_.epp2json(_ns);
    if(_contact.length)
        json.contact = contact_.epp2json(_contact);
    if(_status.length)
        json.status = status_.epp2json(_status);
    return json;
}

function json2addRem(json) {
    let addRem = [];
    if(json.ns!==undefined)
        addRem.push(ns_.json2epp(json));
    if(json.contact!==undefined)
        addRem.push(...contact_.json2epp(json.contact));
    if(json.status instanceof Array)
        addRem.push(...status_.json2epp(json));
    return addRem.length ? addRem : undefined;
}

function chg2json(node) {
    let _registrant = undefined, _authInfo = undefined,
        chg = element.array(node);
    for(let i=0; i<chg.length; i++)
        switch(chg[i][0]) {
        case 'domain:registrant':
            element.limit(chg[i], _registrant!==undefined);
            _registrant = chg[i];
            break;
        case 'domain:authInfo':
            element.limit(chg[i], _authInfo!==undefined);
            _authInfo = chg[i];
            break;
        default:
            throw new EppError(chg[i], chg[i][0]+': unknown');
        }

    let json = {};
    if(_registrant!==undefined)
        json.registrant = token("{0,16}").epp2json(_registrant);
    if(_authInfo!==undefined)
        json.authInfo = authInfo_.epp2json(_authInfo);
    return json;
}

function json2chg(json) {
    let chg = [];
    if(json.registrant!==undefined)
        chg.push(['domain:registrant', {}, token("{0,16}").json2epp(json, 'registrant')]);
    if(json.authInfo!==undefined)
        chg.push(authInfo_.json2epp(json.authInfo));
    return chg.length ? chg : undefined;
}

function epp2json(node) {
    let _name = undefined, _add = undefined, _rem = undefined, _chg = undefined,
        update = element.array(node);
    for(let i=0; i<update.length; i++)
        switch(update[i][0]) {
        case 'domain:name':
            element.limit(update[i], _name!==undefined);
            _name = update[i];
            break;
        case 'domain:add':
            element.limit(update[i], _add!==undefined);
            _add = update[i];
            break;
        case 'domain:rem':
            element.limit(update[i], _rem!==undefined);
            _rem = update[i];
            break;
        case 'domain:chg':
            element.limit(update[i], _chg!==undefined);
            _chg = update[i];
            break;
        default:
            throw new EppError(update[i], update[i][0]+': unknown');
        }
    element.exist(node, 'domain:name', _name!==undefined);

    let json = {
        command: 'update',
        obj: 'domain',
        name: label_.epp2json(_name)
    }
    if(_add!==undefined)
        json.add = addRem2json(_add);
    if(_rem!==undefined)
        json.rem = addRem2json(_rem);
    if(_chg!==undefined)
        json.chg = _chg[2]!==undefined ? chg2json(_chg) : {};
    return json;
}

function json2epp(json) {
    let update = [
        ['domain:name', {}, label_.json2epp(json, 'name')]
    ]
    if(json.add!==undefined)
        update.push(['domain:add', {}, json2addRem(json.add)]);
    if(json.rem!==undefined)
        update.push(['domain:rem', {}, json2addRem(json.rem)]);
    if(json.chg!==undefined)
        update.push(['domain:chg', {}, json2chg(json.chg)]);
    return update;
}

module.exports = {
    epp2json,
    json2epp: json => ["domain:update", {"xmlns:domain": svcNS.domain}, json2epp(json)]
}
