const svcNS    = require('../../definition.js').svcNS;
const EppError = require('../../common/EppError.js');
const element  = require('../../common/element.js');
const label_   = require('../../common/label.js');
const addr_    = require('../../common/host/addr.js');
const status_  = require('./status.js');

function addRem2json(node) {
    let _addr = [], _status = [],
        addRem = element.array(node);
    for(let i=0; i<addRem.length; i++)
        switch(addRem[i][0]) {
        case 'host:addr':
            _addr.push(addRem[i]);
            break;
        case 'host:status':
            _status.push(addRem[i]);
            break;
        default:
            throw new EppError(addRem[i], addRem[i][0]+': unknown');
        }

    let json = {};
    if(_addr.length)
        json.addr = addr_.epp2json(_addr);
    if(_status.length)
        json.status = status_.epp2json(_status);
    return json;
}

function json2addRem(json) {
    let addRem = [];
    if(json.addr instanceof Array)
        addRem.push(...addr_.json2epp(json));
    if(json.status instanceof Array)
        addRem.push(...status_.json2epp(json));
    return addRem;
}

function chg2json(chg) {
    let name = element.array(chg, [1])[0];
    return {name: label_.epp2json(element.tag(name, ['host:name']))}
}

function json2chg(json) {
    return [['host:name', {}, label_.json2epp(json, 'name')]]
}

function epp2json(node) {
    let _name = undefined, _add = undefined, _rem = undefined, _chg = undefined,
        update = element.array(node);
    for(let i=0; i<update.length; i++)
        switch(update[i][0]) {
        case 'host:name':
            element.limit(update[i], _name!==undefined);
            _name = update[i];
            break;
        case 'host:add':
            element.limit(update[i], _add!==undefined);
            _add = update[i];
            break;
        case 'host:rem':
            element.limit(update[i], _rem!==undefined);
            _rem = update[i];
            break;
        case 'host:chg':
            element.limit(update[i], _chg!==undefined);
            _chg = update[i];
            break;
        default:
            throw new EppError(update[i], update[i][0]+': unknown');
        }
    element.exist(node, 'host:name', _name!==undefined);

    let json = {
        command: 'update',
        obj: 'host',
        name: label_.epp2json(_name)
    }
    if(_add!==undefined)
        json.add = addRem2json(_add);
    if(_rem!==undefined)
        json.rem = addRem2json(_rem);
    if(_chg!==undefined)
        json.chg = chg2json(_chg);
    return json;
}

function json2epp(json) {
    let update = [
        ['host:name', {}, label_.json2epp(json, 'name')]
    ]
    if(json.add!==undefined)
        update.push(['host:add', {}, json2addRem(json.add)]);
    if(json.rem!==undefined)
        update.push(['host:rem', {}, json2addRem(json.rem)]);
    if(json.chg!==undefined)
        update.push(['host:chg', {}, json2chg(json.chg)]);
    return update;
}

module.exports = {
    epp2json,
    json2epp: json => ["host:update", {"xmlns:host": svcNS.host}, json2epp(json)]
}
