const definition = require('../definition.js');
const EppError   = require('../common/EppError.js');
const element    = require('../common/element.js');
const token      = require('../common/token.js');

function epp2json(node) {
    let _clID = undefined, _pw = undefined, _newPW = undefined,
        _options = undefined, _svcs = undefined,
        login = element.array(node);
    for(let i=0; i<login.length; i++)
        switch(login[i][0]) {
        case 'clID':
            element.limit(login[i], _clID!==undefined);
            _clID = login[i];
            break;
        case 'pw':
            element.limit(login[i], _pw!==undefined);
            _pw = login[i];
            break;
        case 'newPW':
            element.limit(login[i], _newPW!==undefined);
            _newPW = login[i];
            break;
        case 'options':
            element.limit(login[i], _options!==undefined);
            _options = login[i];
            break;
        case 'svcs':
            element.limit(login[i], _svcs!==undefined);
            _svcs = login[i];
            break;
        default:
            throw new EppError(login[i], login[i][0]+': unknown');
        }
    element.exist(node, 'clID', _clID!==undefined);
    element.exist(node, 'pw', _pw!==undefined);
    element.exist(node, 'options', _options!==undefined);
    element.exist(node, 'svcs', _svcs!==undefined);

    let json = {
        command: "login",
        clID: token("{3,16}").epp2json(_clID),
        pw: token("{6,16}").epp2json(_pw)
    }
    if(_newPW!==undefined)
        json.newPW = token("{6,16}").epp2json(_newPW);
    return json;
}

function json2epp(json) {
    let login = [
        ["clID", {}, token("{3,16}").json2epp(json, 'clID')],
        ["pw", {}, token("{6,16}").json2epp(json, 'pw')]
    ]
    if(json.newPW!==undefined)
        login.push(["newPW", {}, token("{6,16}").json2epp(json, 'newPW')]);
    login.push(
        ["options", {}, [
            ["version", {}, definition.version],
            ["lang", {}, definition.lang]
        ]],
        ["svcs", {}, definition.svcURI]
    );
    return login;
}

module.exports = {
    epp2json,
    json2epp: json => ["login", {}, json2epp(json)]
}
