const EppError = require('../EppError.js');
const element  = require('../element.js');
const string   = require('../string.js');
const token    = require('../token.js');

function epp2json(node) {
    let street = [], city = undefined, sp = undefined, pc = undefined, cc = undefined,
        addr = element.array(node);
    for(let i=0; i<addr.length; i++)
        switch(addr[i][0]) {
        case 'contact:street':
            element.limit(addr[i], street.length, 3);
            street.push(string("{0,255}").epp2json(addr[i]));
            break;
        case 'contact:city':
            element.limit(addr[i], city!==undefined);
            city = string("{1,255}").epp2json(addr[i]);
            break;
        case 'contact:sp':
            element.limit(addr[i], sp!==undefined);
            sp = string("{0,255}").epp2json(addr[i]);
            break;
        case 'contact:pc':
            element.limit(addr[i], pc!==undefined);
            pc = token("{0,16}").epp2json(addr[i]);
            break;
        case 'contact:cc':
            element.limit(addr[i], cc!==undefined);
            cc = token("{2}").epp2json(addr[i]);
            break;
        default:
            throw new EppError(addr[i], addr[i][0]+': unknown');
        }

    let json = {}
    if(street.length)
        json.street = street;
    if(city!==undefined)
        json.city = city;
    if(sp!==undefined)
        json.sp = sp;
    if(pc!==undefined)
        json.pc = pc;
    if(cc!==undefined)
        json.cc = cc;
    return json;
}

function json2epp(json) {
    let addr = [];
    if(json.street instanceof Array)
        for(let i=0; i<json.street.length; i++)
            addr.push(['contact:street', {}, string("{0,255}").json2epp(json, 'street', i)]);
    if(json.city!==undefined)
        addr.push(['contact:city', {}, string("{1,255}").json2epp(json, 'city')]);
    if(json.sp!==undefined)
        addr.push(['contact:sp', {}, string("{0,255}").json2epp(json, 'sp')]);
    if(json.pc!==undefined)
        addr.push(['contact:pc', {}, token("{0,16}").json2epp(json, 'pc')]);
    if(json.cc!==undefined)
        addr.push(['contact:cc', {}, token("{2}").json2epp(json, 'cc')]);
    return addr;
}

module.exports = {
    epp2json,
    json2epp
}
