const EppError = require('../EppError.js');
const element  = require('../element.js');
const bool     = require('../boolean.js');
const TYPETYPE = ['int', 'loc']

function epp2json(node) {
    let name = {}, org = {}, addr = {},
        voice = undefined, fax = undefined, email = undefined;
    for(let j=0; j<node.length; j++) {
        let flag = bool.epp2json(node[j], 'flag'),
            disclose = element.array(node[j]);
        for(let i=0; i<disclose.length; i++)
            switch(disclose[i][0]) {
            case 'contact:name':
                name[element.attribute(disclose[i], 'type', TYPETYPE)] = flag;
                break;
            case 'contact:org':
                org[element.attribute(disclose[i], 'type', TYPETYPE)] = flag;
                break;
            case 'contact:addr':
                addr[element.attribute(disclose[i], 'type', TYPETYPE)] = flag;
                break;
            case 'contact:voice':
                voice = flag;
                break;
            case 'contact:fax':
                fax = flag;
                break;
            case 'contact:email':
                email = flag;
                break;
            default:
                throw new EppError(disclose[i], disclose[i][0]+': unknown');
            }
    }

    let json = {};
    if(Object.keys(name).length)
        json.name = name;
    if(Object.keys(org).length)
        json.org = org;
    if(Object.keys(addr).length)
        json.addr = addr;
    if(voice!==undefined)
        json.voice = voice;
    if(fax!==undefined)
        json.fax = fax;
    if(email!==undefined)
        json.email = email;
    return json;
}

function json2epp(json) {
    let disclose = {false: [], true: []};
    if(json.name!==undefined)
        for(let type in json.name) {
            element.keys(json.name, type, TYPETYPE);
            disclose[bool.json2epp(json, 'name', type)].push(['contact:name', {type}]);
        }
    if(json.org!==undefined)
        for(let type in json.org) {
            element.keys(json.org, type, TYPETYPE);
            disclose[bool.json2epp(json, 'org', type)].push(['contact:org', {type}]);
        }
    if(json.addr!==undefined)
        for(let type in json.addr) {
            element.keys(json.addr, type, TYPETYPE);
            disclose[bool.json2epp(json, 'addr', type)].push(['contact:addr', {type}]);
        }
    if(json.voice!==undefined)
        disclose[bool.json2epp(json, 'voice')].push(['contact:voice', {}]);
    if(json.fax!==undefined)
        disclose[bool.json2epp(json, 'fax')].push(['contact:fax', {}]);
    if(json.email!==undefined)
        disclose[bool.json2epp(json, 'email')].push(['contact:email', {}]);

    let epp = [];
    if(disclose['false'].length)
        epp.push(['contact:disclose', {flag: 'false'}, disclose['false']]);
    if(disclose['true'].length)
        epp.push(['contact:disclose', {flag: 'true'}, disclose['true']]);
    return epp;
}

module.exports = {
    epp2json,
    json2epp
}
