const EppError = require("../EppError.js");
const element  = require("../element.js");
const string   = require("../string.js");
const addr_    = require("./addr.js");
const TYPETYPE = ['int', 'loc']

function postal2json(node) {
    let _name = undefined, _org = undefined, _addr = undefined,
        postal = element.array(node);
    for(let i=0; i<postal.length; i++)
        switch(postal[i][0]) {
        case 'contact:name':
            element.limit(postal[i], _name!==undefined);
            _name = string("{1,255}").epp2json(postal[i]);
            break;
        case 'contact:org':
            element.limit(postal[i], _org!==undefined);
            _org = string("{0,255}").epp2json(postal[i]);
            break;
        case 'contact:addr':
            element.limit(postal[i], _addr!==undefined);
            _addr = addr_.epp2json(postal[i]);
            break;
        default:
            throw new EppError(postal[i], postal[i][0]+': unknown');
        }

    let json = {}
    if(_name!==undefined)
        json.name = _name;
    if(_org!==undefined)
        json.org = _org;
    if(_addr!==undefined)
        json.addr = _addr;
    return json;
}

function json2postal(json) {
    let postal = []
    if(json.name!==undefined)
        postal.push(['contact:name', {}, string("{1,255}").json2epp(json, 'name')]);
    if(json.org!==undefined)
        postal.push(['contact:org', {}, string("{0,255}").json2epp(json, 'org')]);
    if(json.addr!==undefined)
        postal.push(['contact:addr', {}, addr_.json2epp(json.addr)]);
    return postal;
}

function epp2json(postalInfo) {
    let json = {}
    for(let i=0; i<postalInfo.length; i++)
        json[element.attribute(postalInfo[i], 'type', TYPETYPE)] = postal2json(postalInfo[i]);
    return json;
}

function json2epp(json) {
    let postalInfo = []
    for(let type in json) {
        element.keys(json, type, TYPETYPE);
        postalInfo.push(['contact:postalInfo', {type}, json2postal(json[type])]);
    }
    return postalInfo;
}

module.exports = {
    epp2json,
    json2epp
}
