const EppError = require('./EppError.js');

function tag(element, name) {
    if(name.indexOf(element[0]) < 0)
        throw new EppError(element, name.join('|')+': expected');
    return element;
}

function attribute(element, key, name) {
    if(element[1][key]===undefined)
        throw new EppError(element, element[0]+'.'+key+': expected');
    if(name!==undefined && name.indexOf(element[1][key]) < 0)
        throw new EppError(element, element[0]+'.'+key+': '+name.join('|'));
    return element[1][key];
}

function string(element, name) {
    let s = '';
    switch(typeof element[2]) {
    case 'string':
        s = element[2];
    case 'undefined':
        break;
    default:
        throw new EppError(element, element[0]+': expect a string');
    }
    if(name!==undefined && name.indexOf(s) < 0)
        throw new EppError(element, name.join('|')+': expected');
    return s;
}

function array(element, size) {
    if(!(element[2] instanceof Array))
        throw new EppError(element, element[0]+': should have subelement');
    if(size!==undefined && size.indexOf(element[2].length) < 0)
        throw new EppError(element, element[0]+': only have '+size+' subelement');
    return element[2];
}

function limit(element, occur, max = 1) {
    if(occur >= max)
        throw new EppError(element, element[0]+': over '+max+' occur');
}

function exist(element, name, occur) {
    if(occur <= 0)
        throw new EppError(element, name+': expected');
}

function keys(json, key, name) {
    if(name.indexOf(key) < 0)
        throw new EppError(json, name.join('|')+': expected');
}

function value(json, key, name) {
    if(json[key]===undefined)
        throw new EppError(json, key+': expected');
    if(name!==undefined && name.indexOf(json[key]) < 0)
        throw new EppError(json, key+': '+name.join('|'));
    return json[key];
}

function pair(json, key, ...a) {
    let s = json[key], n = key;
    if(s===undefined)
        throw new EppError(json, n+': expected');
    for(let i=0; i<a.length; i++) {
        s = s[a[i]], n += typeof(a[i])==='string' ? '.'+a[i] : '['+a[i]+']';
        if(s===undefined)
            throw new EppError(json, n+': expected');
    }
    return [n, s];
}

module.exports = {
    tag,
    attribute,
    string,
    array,
    limit,
    exist,
    keys,
    value,
    pair
}
