const EppError = require('./EppError.js');
const element  = require('./element.js');
const patt     = /^-?\d+(\.\d*)?$/;

function epp2json(sign, node, key) {
    let s = key===undefined
          ? element.string(node)
          : element.attribute(node, key),
        n = key===undefined
          ? node[0]
          : node[0]+'.'+key;
    if(!patt.test(s))
        throw new EppError(node, n+': '+patt);

    let f = parseFloat(s);
    switch(sign) {
    case '-':
        if(f > 0) throw new EppError(node, n+': nonNegativeDecimal');
        break;
    case '+':
        if(f < 0) throw new EppError(node, n+': negativeDecimal');
        break;
    }
    return f;
}

function json2epp(sign, json, ...args) {
    let [n, f] = element.pair(json, ...args);
    if(typeof(f)!=='number')
        throw new EppError(json, n+': expect a float');
    switch(sign) {
    case '-':
        if(f > 0) throw new EppError(json, n+': nonNegativeDecimal');
        break;
    case '+':
        if(f < 0) throw new EppError(json, n+': negativeDecimal');
        break;
    }

    let s = f.toFixed(2).toString();
    if(!patt.test(s))
        throw new EppError(json, n+': '+patt);
    return s;
}

module.exports = {
    epp2json,
    json2epp
}
