/*
    RFC 4122 - A Universally Unique IDentifier (UUID) URN Namespace
    RFC 5730 - Extensible Provisioning Protocol (EPP)
    RFC 5731 - Extensible Provisioning Protocol (EPP) Domain Name Mapping
    RFC 5732 - Extensible Provisioning Protocol (EPP) Host Mapping
    RFC 5733 - Extensible Provisioning Protocol (EPP) Contact Mapping
    RFC 5734 - Extensible Provisioning Protocol Transport Over TCP
    RFC 3915 - Domain Registry Grace Period Mapping for the EPP
    RFC 5910 - Domain Name System (DNS) Security Extensions Mapping
    RFC 8334 - Launch Phase Mapping for the Extensible Provisioning Protocol (EPP)
    RFC 7848 - Mark and Signed Mark Objects Mapping
    https://www.ietf.org/archive/id/draft-ietf-regext-tmch-func-spec-04.txt
    https://www.ietf.org/archive/id/draft-ietf-regext-epp-fees-14.txt

    https://www.iana.org/assignments/xml-registry/xml-registry.xhtml
    https://tools.ietf.org/html/draft-ietf-regext-allocation-token-12
*/

const objNS = {
    domain : "urn:ietf:params:xml:ns:domain-1.0",
    host   : "urn:ietf:params:xml:ns:host-1.0",
    contact: "urn:ietf:params:xml:ns:contact-1.0"
}
const extNS = {
    rgp    : "urn:ietf:params:xml:ns:rgp-1.0",
    secDNS : "urn:ietf:params:xml:ns:secDNS-1.1",
    launch : "urn:ietf:params:xml:ns:launch-1.0",
    mark   : "urn:ietf:params:xml:ns:mark-1.0",
    smd    : "urn:ietf:params:xml:ns:signedMark-1.0",
    fee    : "urn:ietf:params:xml:ns:epp:fee-1.0"
}
const objURI= Object.keys(objNS).map(ns => ["objURI", {}, objNS[ns]]);
const extURI= Object.keys(extNS).map(ns => ["extURI", {}, extNS[ns]]);

module.exports = {
    version: "1.0",
    lang   : "en",
    svcNS  : Object.assign({}, objNS, extNS),
    svcURI : [...objURI, ["svcExtension", {}, extURI]]
}
