const EppError = require('../../common/EppError.js');
const element  = require('../../common/element.js');
const bool     = require('../../common/boolean.js');
const token    = require('../../common/token.js');
const string   = require('../../common/string.js');
const period_  = require('../../common/domain/period.js');
const fee_     = require('./fee.js');
const credit_  = require('./credit.js');
const COMMAND  = ["create", "delete", "renew", "update", "transfer", "restore", "custom"]

function epp2json(node) {
    let _period = undefined, _fee = [], _credit = [], _reason = undefined;
        data = element.array(node);
    for(let i=0; i<data.length; i++)
        switch(data[i][0]) {
        case 'fee:period':
            element.limit(data[i], _period!==undefined);
            _period = data[i];
            break;
        case 'fee:fee':
            _fee.push(data[i]);
            break;
        case 'fee:credit':
            _credit.push(data[i]);
            break;
        case 'fee:reason':
            element.limit(data[i], _reason!==undefined);
            _reason = data[i];
            break;
        default:
            throw new EppError(data[i], data[i][0]+': unknown');
        }

    let json = {name: element.attribute(node, 'name', COMMAND)}
    if(node[1].customName!==undefined)
        json.customName = token("+").epp2json(node, 'customName');
    if(node[1].phase!==undefined)
        json.phase = token("+").epp2json(node, 'phase');
    if(node[1].subphase!==undefined)
        json.subphase = token("+").epp2json(node, 'subphase');
    if(node[1].standard!==undefined)
        json.standard = bool.epp2json(node, 'standard');
    if(_period!==undefined)
        json.period = period_.epp2json(_period);
    if(_fee.length)
        json.fee = _fee.map(fee_.epp2json);
    if(_credit.length)
        json.credit = _credit.map(credit_.epp2json);
    if(_reason!==undefined)
        json.reason = string("+").epp2json(_reason);
    return json;
}

function json2epp(json) {
    let attrs = {name: element.value(json, 'name', COMMAND)}
    if(json.customName!==undefined)
        attrs.customName = token("+").json2epp(json, 'customName');
    if(json.phase!==undefined)
        attrs.phase = token("+").json2epp(json, 'phase');
    if(json.subphase!==undefined)
        attrs.subphase = token("+").json2epp(json, 'subphase');
    if(json.standard!==undefined)
        attrs.standard = bool.json2epp(json, 'standard');

    let data = [];
    if(json.period!==undefined)
        data.push(period_.json2epp('fee', json.period));
    if(json.fee instanceof Array)
        data.push(...json.fee.map(fee_.json2epp));
    if(json.credit instanceof Array)
        data.push(...json.credit.map(credit_.json2epp));
    if(json.reason!==undefined)
        data.push(['fee:reason', {}, string("+").json2epp(json, 'reason')]);
    return ['fee:command', attrs, data]
}

module.exports = {
    epp2json,
    json2epp
}
