const EppError = require('../../common/EppError.js');
const element  = require('../../common/element.js');
const label_   = require('../../common/label.js');
const bool     = require('../../common/boolean.js');
const token    = require('../../common/token.js');
const string   = require('../../common/string.js');
const command  = require('./commandData.js');

function epp2json(node) {
    let _objID = undefined, _class = undefined, _command = [], _reason = undefined,
        cd = element.array(node);
    for(let i=0; i<cd.length; i++)
        switch(cd[i][0]) {
        case 'fee:objID':
            element.limit(cd[i], _objID!==undefined);
            _objID = cd[i];
            break;
        case 'fee:class':
            element.limit(cd[i], _class!==undefined);
            _class = cd[i];
            break;
        case 'fee:command':
            _command.push(cd[i]);
            break;
        case 'fee:reason':
            element.limit(cd[i], _reason!==undefined);
            _reason = cd[i];
            break;
        default:
            throw new EppError(cd[i], cd[i][0]+': unknown');
        }
    element.exist(node, 'fee:objID', _objID!==undefined);

    let json = {
        avail: bool.epp2json(node, 'avail'),
        objID: label_.epp2json(_objID)
    }
    if(_class!==undefined)
        json.class = token("+").epp2json(_class);
    if(_command.length)
        json.command = _command.map(command.epp2json);
    if(_reason!==undefined)
        json.reason = string("+").epp2json(_reason);
    return json;
}

function json2epp(json) {
    let cd = [['fee:objID', {}, label_.json2epp(json, 'objID')]]
    if(json.class!==undefined)
        cd.push(['fee:class', {}, token("+").json2epp(json, 'class')]);
    if(json.command instanceof Array)
        cd.push(...json.command.map(command.json2epp));
    if(json.reason!==undefined)
        cd.push(['fee:reason', {}, string("+").json2epp(json, 'reason')]);
    return cd;
}

module.exports = {
    epp2json,
    json2epp: json => ['fee:cd', {avail: bool.json2epp(json, 'avail')}, json2epp(json)]
}
