const EppError = require('../../common/EppError.js');
const element  = require('../../common/element.js');
const token    = require('../../common/token.js');
const currency = require('./currency.js');
const fee_     = require('./fee.js');
const credit_  = require('./credit.js');

function epp2json(node) {
    let _class = undefined, _currency = undefined, _fee = [], _credit = [],
        data = element.array(node);
    for(let i=0; i<data.length; i++)
        switch(data[i][0]) {
        case 'fee:class':
            element.limit(data[i], _class!==undefined);
            _class = data[i];
            break;
        case 'fee:currency':
            element.limit(data[i], _currency!==undefined);
            _currency = data[i];
            break;
        case 'fee:fee':
            _fee.push(data[i]);
            break;
        case 'fee:credit':
            _credit.push(data[i]);
            break;
        default:
            throw new EppError(data[i], data[i][0]+': unknown');
        }
    //element.exist(node, 'fee:fee', _fee.length);

    let json = {}
    if(_class!==undefined)
        json.class = token("+").epp2json(_class);
    if(_currency!==undefined)
        json.currency = currency.epp2json(_currency);
    if(_fee.length)
        json.fee = _fee.map(fee_.epp2json);
    if(_credit.length)
        json.credit = _credit.map(credit_.epp2json);
    return json;
}

function json2epp(json) {
    let data = []
    if(json.class!==undefined)
        data.push(['fee:class', {}, token("+").json2epp(json, 'class')]);
    if(json.currency!==undefined)
        data.push(['fee:currency', {}, currency.json2epp(json, 'currency')]);
    if(json.fee instanceof Array)
        data.push(...json.fee.map(fee_.json2epp));
    if(json.credit instanceof Array)
        data.push(...json.credit.map(credit_.json2epp));
    return data;
}

module.exports = {
    epp2json,
    json2epp
}
