const EppError = require('../../common/EppError.js');
const element  = require('../../common/element.js');
const decimal  = require('../../common/float.js');
const period_  = require('../../common/domain/period.js');
const currency = require('./currency.js');
const fee_     = require('./fee.js');
const credit_  = require('./credit.js');

function epp2json(node) {
    let _currency = undefined, _period = undefined, _fee = [], _credit = [],
        _balance = undefined, _creditLimit = undefined,
        data = element.array(node);
    for(let i=0; i<data.length; i++)
        switch(data[i][0]) {
        case 'fee:currency':
            element.limit(data[i], _currency!==undefined);
            _currency = data[i];
            break;
        case 'fee:period':
            element.limit(data[i], _period!==undefined);
            _period = data[i];
            break;
        case 'fee:fee':
            _fee.push(data[i]);
            break;
        case 'fee:credit':
            _credit.push(data[i]);
            break;
        case 'fee:balance':
            element.limit(data[i], _balance!==undefined);
            _balance = data[i];
            break;
        case 'fee:creditLimit':
            element.limit(data[i], _creditLimit!==undefined);
            _creditLimit = data[i];
            break;
        default:
            throw new EppError(data[i], data[i][0]+': unknown');
        }

    let json = {}
    if(_currency!==undefined)
        json.currency = currency.epp2json(_currency);
    if(_period!==undefined)
        json.period = period_.epp2json(_period);
    if(_fee.length)
        json.fee = _fee.map(fee_.epp2json);
    if(_credit.length)
        json.credit = _credit.map(credit_.epp2json);
    if(_balance!==undefined)
        json.balance = decimal.epp2json('?', _balance);
    if(_creditLimit!==undefined)
        json.creditLimit = decimal.epp2json('?', _creditLimit);
    return json;
}

function json2epp(json) {
    let data = []
    if(json.currency!==undefined)
        data.push(['fee:currency', {}, currency.json2epp(json, 'currency')]);
    if(json.period!==undefined)
        data.push(period_.json2epp('fee', json.period));
    if(json.fee instanceof Array)
        data.push(...json.fee.map(fee_.json2epp));
    if(json.credit instanceof Array)
        data.push(...json.credit.map(credit_.json2epp));
    if(json.balance!==undefined)
        data.push(['fee:balance', {}, decimal.json2epp('?', json, 'balance')]);
    if(json.creditLimit!==undefined)
        data.push(['fee:creditLimit', {}, decimal.json2epp('?', json, 'creditLimit')]);
    return data;
}

module.exports = {
    epp2json,
    json2epp
}
