const svcNS    = require('../../definition.js').svcNS;
const EppError = require('../../common/EppError.js');
const element  = require('../../common/element.js');
const label_   = require('../../common/label.js');
const bool     = require('../../common/boolean.js');
const phase_   = require('./phase.js');
const claimKey_= require('./claimKey.js');

function cd2json(node) {
    let _name = undefined, _claimKey = [],
        cd = element.array(node);
    for(let i=0; i<cd.length; i++)
        switch(cd[i][0]) {
        case 'launch:name':
            element.limit(cd[i], _name!==undefined);
            _name = cd[i];
            break;
        case 'launch:claimKey':
            _claimKey.push(cd[i]);
            break;
        default:
            throw new EppError(cd[i], cd[i][0]+': unknown');
        }
    element.exist(node, 'launch:name', _name!==undefined);

    let json = {
        name: label_.epp2json(_name),
        exists: bool.epp2json(_name, 'exists')
    }
    if(_claimKey.length)
        json.claimKey = claimKey_.epp2json(_claimKey);
    return json;
}

function json2cd(json) {
    let cd = [
        ['launch:name', {exists: bool.json2epp(json, 'exists')}, label_.json2epp(json, 'name')]
    ]
    if(json.claimKey instanceof Array)
        cd.push(...claimKey_.json2epp(json));
    return cd;
}

function epp2json(node) {
    let _phase = undefined, _cd = [],
        chkData = element.array(node);
    for(let i=0; i<chkData.length; i++)
        switch(chkData[i][0]) {
        case 'launch:phase':
            element.limit(chkData[i], _phase!==undefined);
            _phase = chkData[i];
            break;
        case 'launch:cd':
            _cd.push(chkData[i]);
            break;
        default:
            throw new EppError(chkData[i], chkData[i][0]+': unknown');
        }
    element.exist(node, 'launch:cd', _cd.length);

    let json = {}
    if(_phase!==undefined)
        json.phase = phase_.epp2json(_phase);
    json.cd = _cd.map(cd2json);
    return json;
}

function json2epp(json) {
    let chkData = []
    if(json.phase!==undefined)
        chkData.push(phase_.json2epp(json));
    chkData.push(...json.cd.map(_cd => ['launch:cd', {}, json2cd(_cd)]));
    return chkData;
}

module.exports = {
    epp2json,
    json2epp: (json, key) => ["launch:chkData", {"xmlns:launch": svcNS.launch}, json2epp(json[key])]
}
