const svcNS    = require('../../definition.js').svcNS;
const EppError = require('../../common/EppError.js');
const element  = require('../../common/element.js');
const phase_   = require('./phase.js');
const code_    = require('./code.js');
const notice_  = require('./notice.js');
const TYPETYPE = ["application", "registration"]

function codeMark2json(node) {
    let json = {},
        codeMark = element.array(node);
    for(let i=0; i<codeMark.length; i++)
        switch(codeMark[i][0]) {
        case 'launch:code':
            element.limit(codeMark[i], json.code!==undefined);
            json.code = code_.epp2json(codeMark[i]);
            break;
        case 'mark:mark':
            element.limit(codeMark[i], json.mark!==undefined);
            json.mark = codeMark[i][2];
            break;
        default:
            throw new EppError(codeMark[i], codeMark[i][0]+': unknown');
        }
    return json;
}

function json2codeMark(json) {
    let codeMark = []
    if(json.code!==undefined)
        codeMark.push(code_.json2epp(json));
    if(json.mark!==undefined)
        codeMark.push(['mark:mark', {"xmlns:mark": svcNS.mark}, json.mark]);
    return ['launch:codeMark', {}, codeMark]
}

function signedMark2json(signedMark) {
    return [signedMark[2], element.attribute(signedMark, 'id')]
}

function json2signedMark(json) {
    return ['smd:signedMark', {"xmlns:smd": svcNS.smd, id: json[1]}, json[0]]
}

function encodedSignedMark2json(encodedSignedMark) {
    return element.string(encodedSignedMark);
}

function json2encodedSignedMark(json) {
    return ['smd:encodedSignedMark', {"xmlns:smd": svcNS.smd}, json]
}

function epp2json(node) {
    let json = {}
    if(node[1].type!==undefined)
        json.type = element.attribute(node, 'type', TYPETYPE);

    let _codeMark = [], _signedMark = [], _encodedSignedMark = [], _notice = [],
        create = element.array(node);
    for(let i=0; i<create.length; i++)
        switch(create[i][0]) {
        case 'launch:phase':
            element.limit(create[i], json.phase!==undefined);
            json.phase = phase_.epp2json(create[i]);
            break;
        case 'launch:codeMark':
            _codeMark.push(codeMark2json(create[i]));
            break;
        case 'smd:signedMark':
            _signedMark.push(signedMark2json(create[i]));
            break;
        case 'smd:encodedSignedMark':
            _encodedSignedMark.push(encodedSignedMark2json(create[i]));
            break;
        case 'launch:notice':
            _notice.push(notice_.epp2json(create[i]));
            break;
        default:
            throw new EppError(create[i], create[i][0]+': unknown');
        }
    element.exist(node, 'launch:phase', json.phase!==undefined);
    if(_codeMark.length)
        json.codeMark = _codeMark;
    if(_signedMark.length)
        json.signedMark = _signedMark;
    if(_encodedSignedMark.length)
        json.encodedSignedMark = _encodedSignedMark;
    if(_notice.length)
        json.notice = _notice;
    return json;
}

function json2epp(json) {
    let attrs = {'xmlns:launch': svcNS.launch}
    if(json.type!==undefined)
        attrs.type = element.value(json, 'type', TYPETYPE);

    let create = [phase_.json2epp(json)]
    if(json.codeMark instanceof Array)
        create.push(...json.codeMark.map(json2codeMark));
    if(json.signedMark instanceof Array)
        create.push(...json.signedMark.map(json2signedMark));
    if(json.encodedSignedMark instanceof Array)
        create.push(...json.encodedSignedMark.map(json2encodedSignedMark));
    if(json.notice instanceof Array)
        create.push(...json.notice.map(notice_.json2epp));
    return ['launch:create', attrs, create]
}

module.exports = {
    epp2json,
    json2epp: (json, key) => json2epp(json[key])
}
