const svcNS    = require('../../definition.js').svcNS;
const EppError = require('../../common/EppError.js');
const element  = require('../../common/element.js');
const token    = require('../../common/token.js');
const phase_   = require('./phase.js');
const status_  = require('./status.js');

function epp2json(node) {
    let json = {}, _mark = [],
        infData = element.array(node);
    for(let i=0; i<infData.length; i++)
        switch(infData[i][0]) {
        case 'launch:phase':
            element.limit(infData[i], json.phase!==undefined);
            json.phase = phase_.epp2json(infData[i]);
            break;
        case 'launch:applicationID':
            element.limit(infData[i], json.applicationID!==undefined);
            json.applicationID = token("+").epp2json(infData[i]);
            break;
        case 'launch:status':
            element.limit(infData[i], json.status!==undefined);
            json.status = status_.epp2json(infData[i]);
            break;
        case 'mark:mark':
            _mark.push(infData[i][2]);
            break;
        default:
            throw new EppError(infData[i], infData[i][0]+': unknown');
        }
    element.exist(node, 'launch:phase', json.phase!==undefined);
    if(_mark.length)
        json.mark = _mark;
    return json;
}

function json2epp(json) {
    let infData = [phase_.json2epp(json)]
    if(json.applicationID!==undefined)
        infData.push(['launch:applicationID', {}, token("+").json2epp(json, 'applicationID')]);
    if(json.status!==undefined)
        infData.push(status_.json2epp(json));
    if(json.mark instanceof Array)
        infData.push(...json.mark.map(mark => ['mark:mark', {'xmlns:mark': svcNS.mark}, mark]));
    return infData;
}

module.exports = {
    epp2json,
    json2epp: (json, key) => ["launch:infData", {"xmlns:launch": svcNS.launch}, json2epp(json[key])]
}
