const svcNS    = require('../../definition.js').svcNS;
const EppError = require('../../common/EppError.js');
const element  = require('../../common/element.js');
const bool     = require('../../common/boolean.js');
const token    = require('../../common/token.js');
const phase_   = require('./phase.js');

function epp2json(node) {
    let json = {}
    if(node[1].includeMark!==undefined)
        json.includeMark = bool.epp2json(node, 'includeMark');

    let info = element.array(node);
    for(let i=0; i<info.length; i++)
        switch(info[i][0]) {
        case 'launch:phase':
            element.limit(info[i], json.phase!==undefined);
            json.phase = phase_.epp2json(info[i]);
            break;
        case 'launch:applicationID':
            element.limit(info[i], json.applicationID!==undefined);
            json.applicationID = token("+").epp2json(info[i]);
            break;
        default:
            throw new EppError(info[i], info[i][0]+': unknown');
        }
    element.exist(node, 'launch:phase', json.phase!==undefined);
    return json;
}

function json2epp(json) {
    let attrs = {'xmlns:launch': svcNS.launch}
    if(json.includeMark!==undefined)
        attrs.includeMark = bool.json2epp(json, 'includeMark');

    let info = [phase_.json2epp(json)]
    if(json.applicationID!==undefined)
        info.push(['launch:applicationID', {}, token("+").json2epp(json, 'applicationID')]);
    return ['launch:info', attrs, info]
}

module.exports = {
    epp2json,
    json2epp: (json, key) => json2epp(json[key])
}
