const svcNS    = require('../../definition.js').svcNS;
const EppError = require('../../common/EppError.js');
const element  = require('../../common/element.js');
const string   = require('../../common/string.js');
const dateTime = require('../../common/dateTime.js');
const OPTYPE   = ['request', 'report']

function report2json(node) {
    let report = element.array(element.tag(node, ['rgp:report']), [6, 7, 8]);
    let json = {
        preData: string("*").epp2json(element.tag(report[0], ['rgp:preData'])),
        postData: string("*").epp2json(element.tag(report[1], ['rgp:postData'])),
        delTime: dateTime.epp2json(element.tag(report[2], ['rgp:delTime'])),
        resTime: dateTime.epp2json(element.tag(report[3], ['rgp:resTime'])),
        resReason: string("*").epp2json(element.tag(report[4], ['rgp:resReason']))
    }
    let _statement = [], _other = undefined;
    for(let i=5; i<report.length; i++)
        switch(report[i][0]) {
        case 'rgp:statement':
            element.limit(report[i], _statement.length, 2);
            _statement.push(string("*").epp2json(report[i]));
            break;
        case 'rgp:other':
            element.limit(report[i], _other!==undefined);
            _other = string("*").epp2json(report[i]);
            break;
        default:
            throw new EppError(report[i], report[i][0]+': unknown');
        }
    element.exist(node, 'rgp:statement', _statement.length);
    json.statement = _statement;
    if(_other!==undefined)
        json.other = _other;
    return json;
}

function json2report(json) {
    let report = [
        ['rgp:preData', {}, string("*").json2epp(json, 'preData')],
        ['rgp:postData', {}, string("*").json2epp(json, 'postData')],
        ['rgp:delTime', {}, dateTime.json2epp(json, 'delTime')],
        ['rgp:resTime', {}, dateTime.json2epp(json, 'resTime')],
        ['rgp:resReason', {}, string("*").json2epp(json, 'resReason')]
    ]
    for(let i=0; i<element.value(json, 'statement').length; i++)
        report.push(['rgp:statement', {}, string("*").json2epp(json, 'statement', i)]);
    if(json.other!==undefined)
        report.push(['rgp:other', {}, string("*").json2epp(json, 'other')]);
    return [['rgp:report', {}, report]]
}

function epp2json(node) {
    switch(element.attribute(element.tag(node, ['rgp:restore']), 'op', OPTYPE)) {
    case "request":
        return {op: "request"}
    case "report":
        return {op: "report", report: report2json(element.array(node, [1])[0])}
    }
}

function json2epp(json) {
    switch(element.value(json, 'op', OPTYPE)) {
    case "request":
        return [['rgp:restore', {op: "request"}]]
    case "report":
        return [['rgp:restore', {op: "report"}, json2report(json.report)]]
    }
}

module.exports = {
    epp2json: node => epp2json(element.array(node, [1])[0]),
    json2epp: (json, key) => ['rgp:update', {'xmlns:rgp': svcNS.rgp}, json2epp(json[key])]
}
