const EppError = require('../../common/EppError.js');
const element  = require('../../common/element.js');
const unsigned = require('../../common/unsigned.js');
const dsData   = require('./dsData.js');
const keyData  = require('./keyData.js');

function epp2json(node) {
    let _maxSigLife = undefined, _dsData = [], _keyData = [],
        dsOrKey = element.array(node);
    for(let i=0; i<dsOrKey.length; i++)
        switch(dsOrKey[i][0]) {
        case 'secDNS:maxSigLife':
            element.limit(dsOrKey[i], _maxSigLife!==undefined);
            _maxSigLife = unsigned.epp2json(dsOrKey[i]);
            break;
        case 'secDNS:dsData':
            _dsData.push(dsData.epp2json(dsOrKey[i]));
            break;
        case 'secDNS:keyData':
            _keyData.push(keyData.epp2json(dsOrKey[i]));
            break;
        default:
            throw new EppError(dsOrKey[i], dsOrKey[i][0]+': unknown');
        }

    let json = {}
    if(_maxSigLife!==undefined)
        json.maxSigLife = _maxSigLife;
    if(_dsData.length)
        json.dsData = _dsData;
    if(_keyData.length)
        json.keyData = _keyData;
    return json;
}

function json2epp(json) {
    let dsOrKey = [];
    if(json.maxSigLife!==undefined)
        dsOrKey.push(['secDNS:maxSigLife', {}, unsigned.json2epp(json, 'maxSigLife')]);
    if(json.dsData instanceof Array)
        dsOrKey.push(...json.dsData.map(_dsData => ['secDNS:dsData', {}, dsData.json2epp(_dsData)]));
    if(json.keyData instanceof Array)
        dsOrKey.push(...json.keyData.map(_keyData => ['secDNS:keyData', {}, keyData.json2epp(_keyData)]));
    return dsOrKey;
}

module.exports = {
    epp2json,
    json2epp
}
