const svcNS    = require('../../definition.js').svcNS;
const EppError = require('../../common/EppError.js');
const element  = require('../../common/element.js');
const bool     = require('../../common/boolean.js');
const dsData   = require('./dsData.js');
const keyData  = require('./keyData.js');
const dsOrKey  = require('./dsOrKey.js');

function rem2json(node) {
    let _all = undefined, _dsData = [], _keyData = [],
        rem = element.array(node);
    for(let i=0; i<rem.length; i++)
        switch(rem[i][0]) {
        case 'secDNS:all':
            element.limit(rem[i], _all!==undefined);
            _all = bool.epp2json(rem[i]);
            break;
        case 'secDNS:dsData':
            element.limit(rem[i], _dsData.length);
            _dsData.push(dsData.epp2json(rem[i]));
            break;
        case 'secDNS:keyData':
            element.limit(rem[i], _keyData.length);
            _keyData.push(keyData.epp2json(rem[i]));
            break;
        default:
            throw new EppError(rem[i], rem[i][0]+': unknown');
        }

    let json = {}
    if(_all!==undefined)
        json.all = _all;
    if(_dsData.length)
        json.dsData = _dsData;
    if(_keyData.length)
        json.keyData = _keyData;
    return json;
}

function json2rem(json) {
    let rem = []
    if(json.all!==undefined)
        rem.push(['secDNS:all', {}, bool.json2epp(json, 'all')]);
    if(json.dsData instanceof Array)
        rem.push(...json.dsData.map(_dsData => ['secDNS:dsData', {}, dsData.json2epp(_dsData)]));
    if(json.keyData instanceof Array)
        rem.push(...json.keyData.map(_keyData => ['secDNS:keyData', {}, keyData.json2epp(_keyData)]));
    return rem;
}

function epp2json(node) {
    let json = {},
        update = element.array(node);
    for(let i=0; i<update.length; i++)
        switch(update[i][0]) {
        case 'secDNS:rem':
            element.limit(update[i], json.rem!==undefined);
            json.rem = rem2json(update[i]);
            break;
        case 'secDNS:add':
            element.limit(update[i], json.add!==undefined);
            json.add = dsOrKey.epp2json(update[i]);
            break;
        case 'secDNS:chg':
            element.limit(update[i], json.chg!==undefined);
            json.chg = dsOrKey.epp2json(update[i]);
            break;
        default:
            throw new EppError(update[i], update[i][0]+': unknown');
        }
    return json;
}

function json2epp(json) {
    let update = []
    if(json.rem!==undefined)
        update.push(['secDNS:rem', {}, json2rem(json.rem)]);
    if(json.add!==undefined)
        update.push(['secDNS:add', {}, dsOrKey.json2epp(json.add)]);
    if(json.chg!==undefined)
        update.push(['secDNS:chg', {}, dsOrKey.json2epp(json.chg)]);
    return update;
}

module.exports = {
    epp2json,
    json2epp: (json, key) => ['secDNS:update', {'xmlns:secDNS': svcNS.secDNS}, json2epp(json[key])]
}
