const svcNS    = require('../../definition.js').svcNS;
const EppError = require('../../common/EppError.js');
const element  = require('../../common/element.js');
const token    = require('../../common/token.js');
const bool     = require('../../common/boolean.js');
const string   = require('../../common/string.js');

function cd2json(node) {
    let _id = undefined, _reason = undefined,
        cd = element.array(node);
    for(let i=0; i<cd.length; i++)
        switch(cd[i][0]) {
        case 'contact:id':
            element.limit(cd[i], _id!==undefined);
            _id = cd[i];
            break;
        case 'contact:reason':
            element.limit(cd[i], _reason!==undefined);
            _reason = cd[i];
            break;
        default:
            throw new EppError(cd[i], cd[i][0]+': unknown');
        }
    element.exist(node, 'contact:id', _id!==undefined);

    let json = {
        id: token("{3,16}").epp2json(_id),
        avail: bool.epp2json(_id, 'avail')
    }
    if(_reason!==undefined)
        json.reason = string("{1,32}").epp2json(_reason);
    return json;
}

function epp2json(node) {
    return {
        obj: 'contact',
        type: 'chkData',
        cd: element.array(node).map(cd => cd2json(element.tag(cd, ['contact:cd'])))
    }
}

function json2cd(json) {
    let cd = [
        ['contact:id', {avail: bool.json2epp(json, 'avail')}, token("{3,16}").json2epp(json, 'id')]
    ]
    if(json.reason!==undefined)
        cd.push(['contact:reason', {}, string("{1,32}").json2epp(json, 'reason')]);
    return cd;
}

function json2epp(json) {
    return json.map(cd => ['contact:cd', {}, json2cd(cd)]);
}

module.exports = {
    epp2json,
    json2epp: json => ["contact:chkData", {"xmlns:contact": svcNS.contact}, json2epp(element.value(json, 'cd'))]
}
