const svcNS      = require('../../definition.js').svcNS;
const EppError   = require('../../common/EppError.js');
const element    = require('../../common/element.js');
const token      = require('../../common/token.js');
const roid       = require('../../common/roid.js');
const dateTime   = require('../../common/dateTime.js');
const postalInfo_= require('../../common/contact/postalInfo.js');
const e164_      = require('../../common/contact/e164.js');
const authInfo_  = require('../../common/contact/authInfo.js');
const disclose_  = require('../../common/contact/disclose.js');
const status_    = require('./status.js');

function epp2json(node) {
    let _id = undefined, _roid = undefined, _status = [], _postalInfo = [],
        _voice = undefined, _fax = undefined, _email = undefined,
        _clID = undefined, _crID = undefined, _crDate = undefined,
        _upID = undefined, _upDate = undefined, _trDate = undefined,
        _authInfo = undefined, _disclose = [],
        infData = element.array(node);
    for(let i=0; i<infData.length; i++)
        switch(infData[i][0]) {
        case 'contact:id':
            element.limit(infData[i], _id!==undefined);
            _id = infData[i];
            break;
        case 'contact:roid':
            element.limit(infData[i], _roid!==undefined);
            _roid = infData[i];
            break;
        case 'contact:status':
            element.limit(infData[i], _status.length, 7);
            _status.push(infData[i]);
            break;
        case 'contact:postalInfo':
            element.limit(infData[i], _postalInfo.length, 2);
            _postalInfo.push(infData[i]);
            break;
        case 'contact:voice':
            element.limit(infData[i], _voice!==undefined);
            _voice = infData[i];
            break;
        case 'contact:fax':
            element.limit(infData[i], _fax!==undefined);
            _fax = infData[i];
            break;
        case 'contact:email':
            element.limit(infData[i], _email!==undefined);
            _email = infData[i];
            break;
        case 'contact:clID':
            element.limit(infData[i], _clID!==undefined);
            _clID = infData[i];
            break;
        case 'contact:crID':
            element.limit(infData[i], _crID!==undefined);
            _crID = infData[i];
            break;
        case 'contact:crDate':
            element.limit(infData[i], _crDate!==undefined);
            _crDate = infData[i];
            break;
        case 'contact:upID':
            element.limit(infData[i], _upID!==undefined);
            _upID = infData[i];
            break;
        case 'contact:upDate':
            element.limit(infData[i], _upDate!==undefined);
            _upDate = infData[i];
            break;
        case 'contact:trDate':
            element.limit(infData[i], _trDate!==undefined);
            _trDate = infData[i];
            break;
        case 'contact:authInfo':
            element.limit(infData[i], _authInfo!==undefined);
            _authInfo = infData[i];
            break;
        case 'contact:disclose':
            element.limit(infData[i], _disclose.length, 2);
            _disclose.push(infData[i]);
            break;
        default:
            throw new EppError(infData[i], infData[i][0]+': unknown');
        }
    element.exist(node, 'contact:id', _id!==undefined);
    element.exist(node, 'contact:roid', _roid!==undefined);
    element.exist(node, 'contact:clID', _clID!==undefined);
    element.exist(node, 'contact:crID', _crID!==undefined);
    element.exist(node, 'contact:crDate', _crDate!==undefined);

    let json = {
        obj : 'contact',
        type: 'infData',
        id: token("{3,16}").epp2json(_id),
        roid: roid.epp2json(_roid)
    }
    if(_status.length)
        json.status = status_.epp2json(_status);
    if(_postalInfo.length)
        json.postalInfo = postalInfo_.epp2json(_postalInfo);
    if(_voice!==undefined)
        json.voice = e164_.epp2json(_voice);
    if(_fax!==undefined)
        json.fax = e164_.epp2json(_fax);
    if(_email!==undefined)
        json.email = token("+").epp2json(_email);
    json.clID = token("{3,16}").epp2json(_clID);
    json.crID = token("{3,16}").epp2json(_crID);
    json.crDate = dateTime.epp2json(_crDate);
    if(_upID!==undefined)
        json.upID = token("{3,16}").epp2json(_upID);
    if(_upDate!==undefined)
        json.upDate = dateTime.epp2json(_upDate);
    if(_trDate!==undefined)
        json.trDate = dateTime.epp2json(_trDate);
    if(_authInfo!==undefined)
        json.authInfo = authInfo_.epp2json(_authInfo);
    if(_disclose.length)
        json.disclose = disclose_.epp2json(_disclose);
    return json;
}

function json2epp(json) {
    let infData = [
        ['contact:id', {}, token("{3,16}").json2epp(json, 'id')],
        ['contact:roid', {}, roid.json2epp(json, 'roid')]
    ]
    if(json.status instanceof Array)
        infData.push(...status_.json2epp(json));
    if(json.postalInfo!==undefined)
        infData.push(...postalInfo_.json2epp(json.postalInfo));
    if(json.voice!==undefined)
        infData.push(e164_.json2epp(json, 'voice'));
    if(json.fax!==undefined)
        infData.push(e164_.json2epp(json, 'fax'));
    if(json.email!==undefined)
        infData.push(['contact:email', {}, token("+").json2epp(json, 'email')]);
    infData.push(['contact:clID', {}, token("{3,16}").json2epp(json, 'clID')]);
    infData.push(['contact:crID', {}, token("{3,16}").json2epp(json, 'crID')]);
    infData.push(['contact:crDate', {}, dateTime.json2epp(json, 'crDate')]);
    if(json.upID!==undefined)
        infData.push(['contact:upID', {}, token("{3,16}").json2epp(json, 'upID')]);
    if(json.upDate!==undefined)
        infData.push(['contact:upDate', {}, dateTime.json2epp(json, 'upDate')]);
    if(json.trDate!==undefined)
        infData.push(['contact:trDate', {}, dateTime.json2epp(json, 'trDate')]);
    if(json.authInfo!==undefined)
        infData.push(authInfo_.json2epp(json.authInfo));
    if(json.disclose!==undefined)
        infData.push(...disclose_.json2epp(json.disclose));
    return infData;
}

module.exports = {
    epp2json,
    json2epp: json => ["contact:infData", {"xmlns:contact": svcNS.contact}, json2epp(json)]
}
