const svcNS    = require('../../definition.js').svcNS;
const EppError = require('../../common/EppError.js');
const element  = require('../../common/element.js');
const label_   = require('../../common/label.js');
const roid     = require('../../common/roid.js');
const token    = require('../../common/token.js');
const dateTime = require('../../common/dateTime.js');
const contact_ = require('../../common/domain/contact.js');
const ns_      = require('../../common/domain/ns.js');
const authInfo_= require('../../common/domain/authInfo.js');
const status_  = require('./status.js');

function epp2json(node) {
    let _name = undefined, _roid = undefined, _status = [], _registrant = undefined,
        _contact = [], _ns = undefined, _host = [], _clID = undefined,
        _crID = undefined, _crDate = undefined, _upID = undefined, _upDate = undefined,
        _exDate = undefined, _trDate = undefined, _authInfo = undefined,
        infData = element.array(node);
    for(let i=0; i<infData.length; i++)
        switch(infData[i][0]) {
        case 'domain:name':
            element.limit(infData[i], _name!==undefined);
            _name = infData[i];
            break;
        case 'domain:roid':
            element.limit(infData[i], _roid!==undefined);
            _roid = infData[i];
            break;
        case 'domain:status':
            element.limit(infData[i], _status.length, 11);
            _status.push(infData[i]);
            break;
        case 'domain:registrant':
            element.limit(infData[i], _registrant!==undefined);
            _registrant = infData[i];
            break;
        case 'domain:contact':
            _contact.push(infData[i]);
            break;
        case 'domain:ns':
            element.limit(infData[i], _ns!==undefined);
            _ns = infData[i];
            break;
        case 'domain:host':
            _host.push(label_.epp2json(infData[i]));
            break;
        case 'domain:clID':
            element.limit(infData[i], _clID!==undefined);
            _clID = infData[i];
            break;
        case 'domain:crID':
            element.limit(infData[i], _crID!==undefined);
            _crID = infData[i];
            break;
        case 'domain:crDate':
            element.limit(infData[i], _crDate!==undefined);
            _crDate = infData[i];
            break;
        case 'domain:upID':
            element.limit(infData[i], _upID!==undefined);
            _upID = infData[i];
            break;
        case 'domain:upDate':
            element.limit(infData[i], _upDate!==undefined);
            _upDate = infData[i];
            break;
        case 'domain:exDate':
            element.limit(infData[i], _exDate!==undefined);
            _exDate = infData[i];
            break;
        case 'domain:trDate':
            element.limit(infData[i], _trDate!==undefined);
            _trDate = infData[i];
            break;
        case 'domain:authInfo':
            element.limit(infData[i], _authInfo!==undefined);
            _authInfo = infData[i];
            break;
        default:
            throw new EppError(infData[i], infData[i][0]+': unknown');
        }
    element.exist(node, 'domain:name', _name!==undefined);
    element.exist(node, 'domain:roid', _roid!==undefined);
    element.exist(node, 'domain:clID', _clID!==undefined);
    element.exist(node, 'domain:crID', _crID!==undefined);
    element.exist(node, 'domain:crDate', _crDate!==undefined);

    let json = {
        obj : 'domain',
        type: 'infData',
        name: label_.epp2json(_name),
        roid: roid.epp2json(_roid)
    }
    if(_status.length)
        json.status = status_.epp2json(_status);
    if(_registrant!==undefined)
        json.registrant = token("{3,16}").epp2json(_registrant);
    if(_contact.length)
        json.contact = contact_.epp2json(_contact);
    if(_ns!==undefined)
        json.ns = ns_.epp2json(_ns);
    if(_host.length)
        json.host = _host;
    json.clID = token("{3,16}").epp2json(_clID);
    json.crID = token("{3,16}").epp2json(_crID);
    json.crDate = dateTime.epp2json(_crDate);
    if(_upID!==undefined)
        json.upID = token("{3,16}").epp2json(_upID);
    if(_upDate!==undefined)
        json.upDate = dateTime.epp2json(_upDate);
    if(_exDate!==undefined)
        json.exDate = dateTime.epp2json(_exDate);
    if(_trDate!==undefined)
        json.trDate = dateTime.epp2json(_trDate);
    if(_authInfo!==undefined)
        json.authInfo = authInfo_.epp2json(_authInfo);
    return json;
}

function json2epp(json) {
    let infData = [
        ['domain:name', {}, label_.json2epp(json, 'name')],
        ['domain:roid', {}, roid.json2epp(json, 'roid')]
    ]
    if(json.status instanceof Array)
        infData.push(...status_.json2epp(json));
    if(json.registrant!==undefined)
        infData.push(['domain:registrant', {}, token("{3,16}").json2epp(json, 'registrant')]);
    if(json.contact!==undefined)
        infData.push(...contact_.json2epp(json.contact));
    if(json.ns instanceof Array)
        infData.push(ns_.json2epp(json));
    if(json.host instanceof Array)
        for(let i=0; i<json.host.length; i++)
            infData.push(['domain:host', {}, label_.json2epp(json, 'host', i)]);
    infData.push(['domain:clID', {}, token("{3,16}").json2epp(json, 'clID')]);
    infData.push(['domain:crID', {}, token("{3,16}").json2epp(json, 'crID')]);
    infData.push(['domain:crDate', {}, dateTime.json2epp(json, 'crDate')]);
    if(json.upID!==undefined)
        infData.push(['domain:upID', {}, token("{3,16}").json2epp(json, 'upID')]);
    if(json.upDate!==undefined)
        infData.push(['domain:upDate', {}, dateTime.json2epp(json, 'upDate')]);
    if(json.exDate!==undefined)
        infData.push(['domain:exDate', {}, dateTime.json2epp(json, 'exDate')]);
    if(json.trDate!==undefined)
        infData.push(['domain:trDate', {}, dateTime.json2epp(json, 'trDate')]);
    if(json.authInfo!==undefined)
        infData.push(authInfo_.json2epp(json.authInfo));
    return infData;
}

module.exports = {
    epp2json,
    json2epp: json => ["domain:infData", {"xmlns:domain": svcNS.domain}, json2epp(json)]
}
