const svcNS    = require('../../definition.js').svcNS;
const EppError = require('../../common/EppError.js');
const element  = require('../../common/element.js');
const label_   = require('../../common/label.js');
const bool     = require('../../common/boolean.js');
const string   = require('../../common/string.js');

function cd2json(node) {
    let _name = undefined, _reason = undefined,
        cd = element.array(node);
    for(let i=0; i<cd.length; i++)
        switch(cd[i][0]) {
        case 'host:name':
            element.limit(cd[i], _name!==undefined);
            _name = cd[i];
            break;
        case 'host:reason':
            element.limit(cd[i], _reason!==undefined);
            _reason = cd[i];
            break;
        default:
            throw new EppError(cd[i], cd[i][0]+': unknown');
        }
    element.exist(node, 'host:name', _name!==undefined);

    let json = {
        name: label_.epp2json(_name),
        avail: bool.epp2json(_name, 'avail')
    }
    if(_reason!==undefined)
        json.reason = string("{1,32}").epp2json(_reason);
    return json;
}

function epp2json(node) {
    return {
        obj: 'host',
        type: 'chkData',
        cd: element.array(node).map(cd => cd2json(element.tag(cd, ['host:cd'])))
    }
}

function json2cd(json) {
    let cd = [
        ['host:name', {avail: bool.json2epp(json, 'avail')}, label_.json2epp(json, 'name')]
    ]
    if(json.reason!==undefined)
        cd.push(['host:reason', {}, string("{1,32}").json2epp(json, 'reason')]);
    return cd;
}

function json2epp(json) {
    return json.map(cd => ['host:cd', {}, json2cd(cd)]);
}

module.exports = {
    epp2json,
    json2epp: json => ["host:chkData", {"xmlns:host": svcNS.host}, json2epp(element.value(json, 'cd'))]
}
