const svcNS    = require('../../definition.js').svcNS;
const EppError = require('../../common/EppError.js');
const element  = require('../../common/element.js');
const label_   = require('../../common/label.js');
const roid     = require('../../common/roid.js');
const token    = require('../../common/token.js');
const dateTime = require('../../common/dateTime.js');
const addr_    = require('../../common/host/addr.js');
const status_  = require('./status.js');

function epp2json(node) {
    let _name = undefined, _roid = undefined, _status = [], _addr = [],
        _clID = undefined, _crID = undefined, _crDate = undefined,
        _upID = undefined, _upDate = undefined, _trDate = undefined,
        infData = element.array(node);
    for(let i=0; i<infData.length; i++)
        switch(infData[i][0]) {
        case 'host:name':
            element.limit(infData[i], _name!==undefined);
            _name = infData[i];
            break;
        case 'host:roid':
            element.limit(infData[i], _roid!==undefined);
            _roid = infData[i];
            break;
        case 'host:status':
            element.limit(infData[i], _status.length, 7);
            _status.push(infData[i]);
            break;
        case 'host:addr':
            _addr.push(infData[i]);
            break;
        case 'host:clID':
            element.limit(infData[i], _clID!==undefined);
            _clID = infData[i];
            break;
        case 'host:crID':
            element.limit(infData[i], _crID!==undefined);
            _crID = infData[i];
            break;
        case 'host:crDate':
            element.limit(infData[i], _crDate!==undefined);
            _crDate = infData[i];
            break;
        case 'host:upID':
            element.limit(infData[i], _upID!==undefined);
            _upID = infData[i];
            break;
        case 'host:upDate':
            element.limit(infData[i], _upDate!==undefined);
            _upDate = infData[i];
            break;
        case 'host:trDate':
            element.limit(infData[i], _trDate!==undefined);
            _trDate = infData[i];
            break;
        default:
            throw new EppError(infData[i], infData[i][0]+': unknown');
        }
    element.exist(node, 'host:name', _name!==undefined);
    element.exist(node, 'host:roid', _roid!==undefined);
    element.exist(node, 'host:clID', _clID!==undefined);
    element.exist(node, 'host:crID', _crID!==undefined);
    element.exist(node, 'host:crDate', _crDate!==undefined);

    let json = {
        obj : 'host',
        type: 'infData',
        name: label_.epp2json(_name),
        roid: roid.epp2json(_roid)
    }
    if(_status.length)
        json.status = status_.epp2json(_status);
    if(_addr.length)
        json.addr = addr_.epp2json(_addr);
    json.clID = token("{3,16}").epp2json(_clID);
    json.crID = token("{3,16}").epp2json(_crID);
    json.crDate = dateTime.epp2json(_crDate);
    if(_upID!==undefined)
        json.upID = token("{3,16}").epp2json(_upID);
    if(_upDate!==undefined)
        json.upDate = dateTime.epp2json(_upDate);
    if(_trDate!==undefined)
        json.trDate = dateTime.epp2json(_trDate);
    return json;
}

function json2epp(json) {
    let infData = [
        ['host:name', {}, label_.json2epp(json, 'name')],
        ['host:roid', {}, roid.json2epp(json, 'roid')]
    ]
    if(json.status instanceof Array)
        infData.push(...status_.json2epp(json));
    if(json.addr instanceof Array)
        infData.push(...addr_.json2epp(json));
    infData.push(['host:clID', {}, token("{3,16}").json2epp(json, 'clID')]);
    infData.push(['host:crID', {}, token("{3,16}").json2epp(json, 'crID')]);
    infData.push(['host:crDate', {}, dateTime.json2epp(json, 'crDate')]);
    if(json.upID!==undefined)
        infData.push(['host:upID', {}, token("{3,16}").json2epp(json, 'upID')]);
    if(json.upDate!==undefined)
        infData.push(['host:upDate', {}, dateTime.json2epp(json, 'upDate')]);
    if(json.trDate!==undefined)
        infData.push(['host:trDate', {}, dateTime.json2epp(json, 'trDate')]);
    return infData;
}

module.exports = {
    epp2json,
    json2epp: json => ["host:infData", {"xmlns:host": svcNS.host}, json2epp(json)]
}
