const EppError = require('../common/EppError.js');
const element  = require('../common/element.js');
const result   = require('./result.js');
const msgQ     = require('./msgQ.js');
const resData  = require('./resData.js');
const extension= require('./extension.js');
const trID_    = require('./trID.js');

function epp2json(node) {
    let _result = undefined, _msgQ = undefined, _resData = undefined,
        _extension = undefined, _trID = undefined,
        response = element.array(node);
    for(let i=0; i<response.length; i++)
        switch(response[i][0]) {
        case 'result':
            element.limit(response[i], _result!==undefined);
            _result = response[i];
            break;
        case 'msgQ':
            element.limit(response[i], _msgQ!==undefined);
            _msgQ = response[i];
            break;
        case 'resData':
            element.limit(response[i], _resData!==undefined);
            _resData = response[i];
            break;
        case 'extension':
            element.limit(response[i], _extension!==undefined);
            _extension = response[i];
            break;
        case 'trID':
            element.limit(response[i], _trID!==undefined);
            _trID = response[i];
            break;
        default:
            throw new EppError(response[i], response[i][0]+': unknown');
        }
    element.exist(node, 'result', _result!==undefined);
    element.exist(node, 'trID', _trID!==undefined);

    let json = {
        format: "response",
        result: result.epp2json(_result)
    }
    if(_msgQ!==undefined)
        json.msgQ = msgQ.epp2json(_msgQ);
    if(_resData!==undefined)
        json.resData = resData.epp2json(_resData);
    if(_extension!==undefined)
        json.extension = extension.epp2json(json.resData ? json.resData.type : 'resData', _extension);
    json.trID = trID_.epp2json(_trID);
    return json;
}

function json2epp(json) {
    let response = [result.json2epp(element.value(json, 'result'))]
    if(json.msgQ!==undefined)
        response.push(msgQ.json2epp(json.msgQ));
    if(json.resData!==undefined)
        response.push(resData.json2epp(json.resData));
    if(json.extension!==undefined)
        response.push(extension.json2epp(json.resData ? json.resData.type : 'resData', json.extension));
    response.push(['trID', {}, trID_.json2epp(element.value(json, 'trID'))]);
    return response;
}

module.exports = {
    epp2json,
    json2epp: json => ['response', {}, json2epp(json)]
}
