const svcNS    = require('../definition.js').svcNS;
const EppError = require('../common/EppError.js');
const element  = require('../common/element.js');
const unsigned = require('../common/unsigned.js');
const string   = require('../common/string.js');
const CODETYPE = {
    '1000': 'Command completed successfully',
    '1001': 'Command completed successfully; action pending',
    '1300': 'Command completed successfully; no messages',
    '1301': 'Command completed successfully; ack to dequeue',
    '1500': 'Command completed successfully; ending session',
    '2000': 'Unknown command',
    '2001': 'Command syntax error',
    '2002': 'Command use error',
    '2003': 'Required parameter missing',
    '2004': 'Parameter value range error',
    '2005': 'Parameter value syntax error',
    '2100': 'Unimplemented protocol version',
    '2101': 'Unimplemented command',
    '2102': 'Unimplemented option',
    '2103': 'Unimplemented extension',
    '2104': 'Billing failure',
    '2105': 'Object is not eligible for renewal',
    '2106': 'Object is not eligible for transfer',
    '2200': 'Authentication error',
    '2201': 'Authorization error',
    '2202': 'Invalid authorization information',
    '2300': 'Object pending transfer',
    '2301': 'Object not pending transfer',
    '2302': 'Object exists',
    '2303': 'Object does not exist',
    '2304': 'Object status prohibits operation',
    '2305': 'Object association prohibits operation',
    '2306': 'Parameter value policy error',
    '2307': 'Unimplemented object service',
    '2308': 'Data management policy violation',
    '2400': 'Command failed',
    '2500': 'Command failed; server closing connection',
    '2501': 'Authentication error; server closing connection',
    '2502': 'Session limit exceeded; server closing connection'
}
const CODE = Object.keys(CODETYPE);

function extValue2json(node) {
    let extValue = element.array(node, [2]);
    return {
        value: element.array(element.tag(extValue[0], ['value']), [1])[0],
        reason: string("+").epp2json(element.tag(extValue[1], ['reason']))
    }
}

function json2extValue(json) {
    let epp = element.value(json, 'value'),
        tag = epp[0].split(':'), attrs = {};
    if(svcNS[tag[0]]!==undefined)
        attrs["xmlns:"+tag[0]] = svcNS[tag[0]];
    return [
        ["value", attrs, [epp]],
        ["reason", {}, string("+").json2epp(json, 'reason')]
    ]
}

function epp2json(node) {
    let _msg = undefined, _extValue = undefined,
        result = element.array(node);
    for(let i=0; i<result.length; i++)
        switch(result[i][0]) {
        case 'msg':
            element.limit(result[i], _msg!==undefined);
            _msg = result[i];
            break;
        case 'extValue':
            element.limit(result[i], _extValue!==undefined);
            _extValue = result[i];
            break;
        default:
            throw new EppError(result[i], result[i][0]+': unknown');
        }
    element.exist(node, 'msg', _msg!==undefined);

    let json = {
        code: unsigned.epp2json(node, 'code'),
        msg: string("+").epp2json(_msg)
    }
    if(_extValue!==undefined)
        json.extValue = extValue2json(_extValue);
    return json;
}

function json2epp(json) {
    let code = unsigned.json2epp(json, 'code');
    element.keys(json, code, CODE);
    json.msg = json.msg || CODETYPE[code];

    let result = [['msg', {}, string("+").json2epp(json, 'msg')]]
    if(json.extValue!==undefined)
        result.push(['extValue', {}, json2extValue(json.extValue)]);
    return result;
}

module.exports = {
    epp2json,
    json2epp: json => ['result', {code: unsigned.json2epp(json, 'code')}, json2epp(json)]
}
